@@ -1,3 +1,4 @@
+/* $Id: mkzftree.c,v 1.8 2001/05/07 01:27:53 hpa Exp $ */
 /* ----------------------------------------------------------------------- *
  *   
  *   Copyright 2001 H. Peter Anvin - All Rights Reserved
@@ -67,6 +68,7 @@
 #include <limits.h>
 #include <utime.h>
 #include <signal.h>
+#include <stdarg.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -75,12 +77,19 @@
 
 #define HAVE_LCHOWN 1		/* Should be obtained by autoconf or so */
 
+/* Verbosity levels */
+enum verbosity {
+  vl_quiet,			/* No messages */
+  vl_error,			/* Error messages only */
+  vl_filename,			/* Display filenames */
+};
+  
 /* Command line options */
 int force = 0;			/* Always compress */
 int level = 9;			/* Compression level */
 int parallel = 0;		/* Parallelism (0 = strictly serial) */
 int onefs = 0;			/* One filesystem only */
-int verbosity = 0;
+enum verbosity verbosity = vl_error;	/* Default verbosity */
 
 /* Program name */
 const char *program;
@@ -174,6 +183,17 @@ char *xstrdup(const char *str)
   return s;
 }
 
+void message(int level, const char *format, ...)
+{
+  va_list ap;
+
+  va_start(ap, format);
+  if ( verbosity >= level )
+    vfprintf(stderr, format, ap);
+  va_end(ap);
+}
+
+/* iso9660 integer formats */
 static void
 set_721(void *pnt, unsigned int i)
 {
@@ -641,7 +661,7 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
 
   /* Get directory information */
   if ( stat(intree, &dirst) ) {
-    fprintf(stderr, "%s: Failed to stat directory %s: %s\n",
+    message(vl_error, "%s: Failed to stat directory %s: %s\n",
 	    program, intree, strerror(errno));
     return 1;
   }
@@ -649,14 +669,14 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
   /* Open the directory */
   thisdir = opendir(intree);
   if ( !thisdir ) {
-    fprintf(stderr, "%s: Failed to open directory %s: %s\n",
+    message(vl_error, "%s: Failed to open directory %s: %s\n",
 	    program, intree, strerror(errno));
     return 1;
   }
 
   /* Create output directory */
   if ( mkdir(outtree, 0700) ) {
-    fprintf(stderr, "%s: Cannot create output directory %s: %s\n",
+    message(vl_error, "%s: Cannot create output directory %s: %s\n",
 	    program, outtree, strerror(errno));
     return 1;
   }
@@ -669,8 +689,10 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
     strcpy(in_file, dirent->d_name);
     strcpy(out_file, dirent->d_name);
 
+    message(vl_filename, "%s -> %s\n", in_path, out_path);
+
     if ( lstat(in_path, &st) ) {
-      fprintf(stderr, "%s: Failed to stat file %s: %s\n",
+      message(vl_error, "%s: Failed to stat file %s: %s\n",
 	      program, in_path, strerror(errno));
       err = 1;
       break;
@@ -685,7 +707,7 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
 	  /* We've seen it before, hard link it */
 
 	  if ( link(linkname, out_path) ) {
-	    fprintf(stderr, "%s: hard link %s -> %s failed: %s\n",
+	    message(vl_error, "%s: hard link %s -> %s failed: %s\n",
 		    program, out_path, linkname, strerror(errno));
 	    err = 1;
 	    break;
@@ -693,7 +715,7 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
 	} else {
 	  /* First encounter, compress and enter into hash */
 	  if ( munge_path(in_path, out_path, &st, munger) ) {
-	    fprintf(stderr, "%s: %s: %s", program, in_path, strerror(errno));
+	    message(vl_error, "%s: %s: %s", program, in_path, strerror(errno));
 	    err = 1;
 	    break;
 	  }
@@ -702,7 +724,7 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
       } else {
 	/* Singleton file; no funnies */
 	if ( munge_path(in_path, out_path, &st, munger) ) {
-	  fprintf(stderr, "%s: %s: %s", program, in_path, strerror(errno));
+	  message(vl_error, "%s: %s: %s", program, in_path, strerror(errno));
 	  err = 1;
 	  break;
 	}
@@ -718,14 +740,14 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
     } else if ( S_ISLNK(st.st_mode) ) {
       int chars;
       if ( (chars = readlink(in_path, buffer, BUFSIZ)) < 0 ) {
-	fprintf(stderr, "%s: readlink failed for %s: %s\n",
+	message(vl_error, "%s: readlink failed for %s: %s\n",
 		program, in_path, strerror(errno));
 	err = 1;
 	break;
       }
       buffer[chars] = '\0';
       if ( symlink(buffer, out_path) ) {
-	fprintf(stderr, "%s: symlink %s -> %s failed: %s\n",
+	message(vl_error, "%s: symlink %s -> %s failed: %s\n",
 		program, out_path, buffer, strerror(errno));
 	err = 1;
 	break;
@@ -739,7 +761,7 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
 	  /* We've seen it before, hard link it */
 
 	  if ( link(linkname, out_path) ) {
-	    fprintf(stderr, "%s: hard link %s -> %s failed: %s\n",
+	    message(vl_error, "%s: hard link %s -> %s failed: %s\n",
 		    program, out_path, linkname, strerror(errno));
 	    err = 1;
 	    break;
@@ -747,7 +769,7 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
 	} else {
 	  /* First encounter, create and enter into hash */
 	  if ( mknod(out_path, st.st_mode, st.st_rdev) ) {
-	    fprintf(stderr, "%s: mknod failed for %s: %s\n",
+	    message(vl_error, "%s: mknod failed for %s: %s\n",
 		    program, out_path, strerror(errno));
 	    err = 1;
 	    break;
@@ -757,7 +779,7 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
       } else {
 	/* Singleton node; no funnies */
 	  if ( mknod(out_path, st.st_mode, st.st_rdev) ) {
-	    fprintf(stderr, "%s: mknod failed for %s: %s\n",
+	    message(vl_error, "%s: mknod failed for %s: %s\n",
 		    program, out_path, strerror(errno));
 	    err = 1;
 	    break;
@@ -789,10 +811,10 @@ int munge_tree(const char *intree, const char *outtree, munger_func munger)
   return err;
 }
 
-static void usage(int err)
+static void usage(enum verbosity level, int err)
 {
-  fprintf(stderr,
-	  "Usage: %s [-vfhux] [-p parallelism] [-z level] intree outtree\n",
+  message(level,
+	  "Usage: %s [-vqfhux] [-p parallelism] [-z level] intree outtree\n",
 	  program);
   exit(err);
 }
@@ -807,14 +829,14 @@ int main(int argc, char *argv[])
 
   program = argv[0];
 
-  while ( (opt = getopt(argc, argv, "vfz:p:hux")) != EOF ) {
+  while ( (opt = getopt(argc, argv, "vqfz:p:hux")) != EOF ) {
     switch(opt) {
     case 'f':
       force = 1;		/* Always compress */
       break;
     case 'z':
       if ( optarg[0] < '0' || optarg[0] > '9' || optarg[1] ) {
-	fprintf(stderr, "%s: invalid compression level: %s\n",
+	message(vl_error, "%s: invalid compression level: %s\n",
 		program, optarg);
 	exit(1);
       } else {
@@ -822,11 +844,14 @@ int main(int argc, char *argv[])
       }
       break;
     case 'h':
-      usage(0);
+      usage(vl_quiet, 0);
       break;
     case 'v':
       verbosity++;
       break;
+    case 'q':
+      verbosity = vl_quiet;
+      break;
     case 'u':
       munger = block_uncompress_file;
       break;
@@ -837,13 +862,13 @@ int main(int argc, char *argv[])
       onefs = 1;
       break;
     default:
-      usage(1);
+      usage(vl_error, 1);
       break;
     }
   }
 
   if ( (argc-optind) != 2 )
-    usage(1);
+    usage(vl_error, 1);
 
   in  = argv[optind];		/* Input tree */
   out = argv[optind+1];		/* Output tree */
@@ -852,11 +877,12 @@ int main(int argc, char *argv[])
 
   /* Special case: we use stat() for the root, not lstat() */
   if ( stat(in, &st) ) {
-    fprintf(stderr, "%s: %s: %s\n", program, in, strerror(errno));
+    message(vl_error, "%s: %s: %s\n", program, in, strerror(errno));
     exit(1);
   }
   if ( !S_ISDIR(st.st_mode) ) {
-    fprintf(stderr, "%s: %s: Not a directory\n", program, in);
+    message(vl_error, "%s: %s: Not a directory\n", program, in);
+    exit(1);
   }
 
   err = munge_tree(in, out, munger);
